# -*- coding: utf8 -*-

from datetime import datetime

from django.shortcuts import render, get_object_or_404, redirect
from django.utils.safestring import mark_safe

from lib.access_auth import permission_needed
from lib.tools import confirmation, erreur
from portailuser.app_logger import AppLogger

from .forms import PageWebForm
from .models import PageWeb, JetonEditionPage


# Logs
_logger = AppLogger("pages")


@permission_needed("gerer_pages")
def liste(request):
    c = {"pages": PageWeb.objects.filter(ecole=request.ecole).order_by("adresse")}

    return render(request, "pages/liste.html", c)


@permission_needed("gerer_pages")
def edition(request, id=None):
    if id is not None:
        page = get_object_or_404(PageWeb, ecole=request.ecole, id=id)
        try:
            jeton = JetonEditionPage.objects.get(page=page)
        except JetonEditionPage.DoesNotExist:
            jeton = JetonEditionPage(page=page, editeur=request.user)
            jeton.save()
        else:
            if jeton.editeur == request.user:
                jeton.renouvellement()
            elif jeton.fin_jeton < datetime.now():
                jeton.delete()
                jeton = JetonEditionPage(page=page, editeur=request.user)
                jeton.save()
            else:
                msg = mark_safe("{} est en train d'éditer cette page.".format(jeton.editeur.get_link()))
                return erreur(request, msg, "Conflit d'édition", 409)
    else:
        page = None
        jeton = None

    previs = None
    namespace = "static_pages"

    if request.method == "POST":
        form = PageWebForm(None, request.POST, request.FILES, instance=page, namespace=namespace)

        if "previsualisation" in request.POST:
            previs = form.get_preview()
        elif "cancel" in request.POST and jeton:
            jeton.delete()
            return redirect("/" + page.adresse)
        elif form.is_valid():
            page = form.save(commit=False)
            page.ecole = request.ecole
            page.save()
            form.commit_attachments()
            _logger.log(
                request.user,
                "A édité la page « {} » (école {})",
                page.titre,
                page.ecole,
            )

            if jeton:
                jeton.delete()

            return redirect("/" + page.adresse)

    else:
        form = PageWebForm(request.GET.get("adresse", None), instance=page, namespace=namespace)

    c = {
        "page": page,
        "form": form,
        "previs": previs,
    }

    return render(request, "pages/edition.html", c)


@permission_needed("gerer_pages")
def suppression(request, id):
    page = get_object_or_404(PageWeb, ecole=request.ecole, id=id)
    namespace = "static_pages"

    if request.method == "POST":
        if "conf_ok" in request.POST:
            _logger.log(
                request.user,
                "A supprimé la page « {} » (école {})",
                page.titre,
                page.ecole,
            )
            page.contenu.disown_attachments(namespace, page.id)
            page.delete()

        return redirect("gestion:pages:liste")

    return confirmation(
        request,
        f"Êtes-vous sûr de vouloir supprimer la page « {page.titre} » ?",
        [("conf_annuler", "Annuler"), ("conf_ok", "Supprimer")],
        "Gestion des pages Web",
    )
